/*
 * Copyright 2004 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.nordija.tapestry.bayeux.downloads;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.engine.IEngineService;
import org.apache.tapestry.engine.ILink;
import org.apache.tapestry.services.LinkFactory;
import org.apache.tapestry.spec.IApplicationSpecification;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;

/**
 * A service which can render links to download resources and accept incoming requests
 * from such links, looking up the download resource and dispatching to the resource.
 * <p/>
 * This service depends on the definition of a tapestry-extension which must point to a
 * implementation of {@link com.nordija.tapestry.bayeux.downloads.DownloadResourceSource}. Please see docs for further info.
 *
 * @author Per Olesen www.nordija.com
 * @version $Id: DownloadService.java 76 2007-05-14 17:12:15Z jeyben $
 */
public class DownloadService implements IEngineService {

    public static final String SERVICE_NAME = "downloads";

    public static final String RESOURCE_ID_PARAM_NAME = "resourceId";
    public static final String DOWNLOAD_RESOURCE_SOURCE_EXT_PARAM_NAME = "downloadResourceSourceExtentionName";
    public static final String BAYEUX_DOWNLOADS_EXTENTIONS_PREFIX = "bayeux.downloads.";
    public static final String DEFAULT_RESOURCE_SOURCE_NAME = "resourcesource";

    private static final Log LOG = LogFactory.getLog(DownloadService.class);

    private HttpServletResponse response;

    private LinkFactory linkFactory;

    public void setResponse(HttpServletResponse response) {
        this.response = response;
    }

    public void setLinkFactory(LinkFactory linkFactory) {
        this.linkFactory = linkFactory;
    }

    public String getName() {
        return SERVICE_NAME;
    }

    public ILink getLink(boolean post, Object parameter) {
        String resourceId = (String) ((Object[]) parameter)[0];
        String extName = (String) ((Object[]) parameter)[1];
        Map parameters = new HashMap();
        parameters.put("service", getName());
        parameters.put(RESOURCE_ID_PARAM_NAME, resourceId);
        parameters.put(DOWNLOAD_RESOURCE_SOURCE_EXT_PARAM_NAME, extName);
        return linkFactory.constructLink(this, post, parameters, true);
    }

    public void service(IRequestCycle cycle) throws IOException {
        // look into the applications ApplicationSpecification to find the extension that
        // enables lookup of resources by their id.
        IApplicationSpecification specification = cycle.getInfrastructure().getApplicationSpecification();

        // get the id of the resource from the incoming link
        String downloadResourceSourceExtName = cycle.getParameter(DOWNLOAD_RESOURCE_SOURCE_EXT_PARAM_NAME);

        if (downloadResourceSourceExtName == null) {
            if (LOG.isDebugEnabled()) {
                LOG.debug("no '" + DOWNLOAD_RESOURCE_SOURCE_EXT_PARAM_NAME + "' parameter found. Will try to use the default downloadResourceSource '" + DEFAULT_RESOURCE_SOURCE_NAME + "'");
            }
            downloadResourceSourceExtName = DEFAULT_RESOURCE_SOURCE_NAME;
        }

        // ask the extension for the resource
        DownloadResourceSource source = (DownloadResourceSource) specification.getExtension(BAYEUX_DOWNLOADS_EXTENTIONS_PREFIX + downloadResourceSourceExtName, DownloadResourceSource.class);

        // get the id of the resource from the incoming link
        String resourceId = cycle.getParameter(RESOURCE_ID_PARAM_NAME);

        if (resourceId == null) {
            throw new ApplicationRuntimeException("unable to process download link, no '" + RESOURCE_ID_PARAM_NAME + "' parameter found");
        }

        if (LOG.isDebugEnabled()) {
            LOG.debug("going to process download resource with id '" + resourceId + "'");
        }

        // ask the source for the resource to be downloaded
        DownloadResource downloadResource = source.getDownloadResource(cycle, resourceId);
        if (downloadResource == null) {
            throw new ApplicationRuntimeException("unable to process download link, no resource found on id: '" + resourceId + "'");
        }

        streamResource(resourceId, downloadResource);
    }

    void streamResource(String resourceId, DownloadResource resource) throws IOException {
        if (LOG.isDebugEnabled()) {
            LOG.debug("streaming resource '" + resourceId + "' with content-type '" + resource.getContentType() + "'");
        }
        response.setContentType(resource.getContentType());
        String contentDisposition = resource.getContentDisposition();
        if (contentDisposition != null) {
            response.setHeader("Content-disposition", contentDisposition);
        }
        OutputStream out = response.getOutputStream();
        InputStream resourceInput = null;
        try {
            resourceInput = resource.getInputStream();
            byte[] buf = new byte[1024];
            int bytesRead;
            while ((bytesRead = resourceInput.read(buf)) != -1) {
                out.write(buf, 0, bytesRead);
            }
            out.flush();
        } finally {
            if (resourceInput != null) {
                resourceInput.close();
            }
        }
    }
}
