/*
 * Copyright 2004 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.nordija.tapestry.bayeux.form;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tapestry.IActionListener;
import org.apache.tapestry.IBinding;
import org.apache.tapestry.IForm;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.form.AbstractFormComponent;

/**
 * Component which renders a HTML4 button element (&lt;button type="..."&gt;Create&lt;/button&gt;).
 * <p/>
 * The implementation of this component has been heavily inspired by the standard Tapestry Button and Submit components.
 *
 * @author Per Olesen, www.nordija.com
 * @version $Id: Html4Button.java 90 2007-05-18 13:53:39Z jeyben $
 */
public abstract class Html4Button extends AbstractFormComponent {

    /**
     * the value of hidden button control element which is set when button is clicked (workaround on an IE bug)
     */
    public static final String BUTTON_SUBMIT_VALUE = "x";

    private static final Log LOG = LogFactory.getLog(Html4Button.class);

    public abstract String getLabel();

    public abstract String getType();

    public abstract IBinding getSelectedBinding();

    public abstract boolean isDisabled();

    public abstract String getOnclickEventHandler();

    public abstract IActionListener getListener();

    public abstract Object getTag();

    protected void renderFormComponent(IMarkupWriter writer, IRequestCycle cycle) {
        IForm form = getForm();
        String buttonName = form.getElementId(this);
        String formName = form.getName();

        if (LOG.isDebugEnabled()) {
            LOG.debug("form=" + form + ", buttonName=" + buttonName);
        }
        processRendering(cycle, formName, buttonName, writer);
    }

    protected void rewindFormComponent(IMarkupWriter writer, IRequestCycle cycle) {
        IForm form = getForm();
        String buttonName = form.getElementId(this);
       if (LOG.isDebugEnabled()) {
            LOG.debug("rewinding: form=" + form + ", buttonName=" + buttonName);
        }
         processRewinding(cycle, buttonName);
    }

    private void processRendering(IRequestCycle cycle, String formName, String buttonName, IMarkupWriter writer) {
        //
        // A bug in IE6 makes it submit ALL buttons on a form if one of the buttons are clicked
        // which again makes tapestry think all buttons where clicked which results in all listeners
        // being activated.
        //
        // We work around this by putting a hidden input field in form for each submit button, and
        // letting the button set the value of this field if it is clicked, before submit.
        // At rewind, we then dispatch on this hidden value instead of button name.
        //
        writer.begin("input");
        writer.attribute("type", "hidden");
        writer.attribute("name", getHiddenElementName(buttonName));
        writer.end();

        writer.begin("button");
        writer.attribute("type", getType());
        writer.attribute("name", buttonName);

        if (isDisabled()) {
            writer.attribute("disabled", "disabled");
        }

        String label = getLabel();
        if (label != null) {
            writer.attribute("value", label);
        }

        // this is the js code, that sets the hidden element before the button-click submits the form
        String onclickEvent = "document." + formName + "." + getHiddenElementName(buttonName) + ".value='" + BUTTON_SUBMIT_VALUE + "'";

        // if a specific onclick handler was specified, this js code is appended
        if ((getOnclickEventHandler() != null) && (getOnclickEventHandler().trim().length() > 0)) {
            onclickEvent += "; " + getOnclickEventHandler();
        }

        writer.attribute("onclick", onclickEvent);

        renderInformalParameters(writer, cycle);
        renderBody(writer, cycle);
        writer.end();
    }

    private void processRewinding(IRequestCycle cycle, String buttonName) {
        if (isDisabled()) {
            return;
        }

        // As with the standard Submit component, the value of the button is submitted with the
        // form and is used to determine if we should dispatch to listener (ie. if it was this button that was clicked).
        // A bug in IE6 makes it submit all button-elements in a form if one such is clicked, which makes
        // dispatching on the button value break. This is worked around by letting the button set a hidden
        // value in form and use that to determine if we should dispatch.
        String value = cycle.getParameter(getHiddenElementName(buttonName));
        if (value == null || (!value.equals(BUTTON_SUBMIT_VALUE))) {
            return; // button was not clicked, submit of form was from some other component on same form
        }

        if (LOG.isDebugEnabled()) {
            LOG.debug("determined that it was this button '" + buttonName + "' that was clicked");
        }
        // copy tag over into selected paramter
        IBinding selectedBinding = getSelectedBinding();
        if (selectedBinding != null) {
            selectedBinding.setObject(getTag());
        }

        IActionListener listener = getListener();
        if (listener != null) {
            listener.actionTriggered(this, cycle);
        }
    }

    private String getHiddenElementName(String buttonName) {
        return buttonName + "_ctrl";
    }
}