/*
 * Copyright 2004 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.nordija.tapestry.bayeux.stream;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.hivemind.util.Defense;
import org.apache.tapestry.IComponent;
import org.apache.tapestry.IPage;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.StaleSessionException;
import org.apache.tapestry.engine.EngineMessages;
import org.apache.tapestry.engine.IEngineService;
import org.apache.tapestry.engine.ILink;
import org.apache.tapestry.services.LinkFactory;
import org.apache.tapestry.services.ServiceConstants;
import org.apache.tapestry.spec.IApplicationSpecification;
import org.apache.tapestry.web.WebRequest;
import org.apache.tapestry.web.WebSession;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;

/**
 * A service which can render links to stream resources and accept incoming requests
 * from such links, looking up the stream resource and dispatching to the resource.
 * <p/>
 * This service depends on the definition of a tapestry-extension which must point to a
 * implementation of {@link StreamResourceSource}. Please see docs for further info.
 *
 * @author Per Olesen www.nordija.com
 * @version $Id: StreamService.java 124 2007-06-02 11:36:55Z jeyben $
 */
public class StreamService implements IEngineService {

    public static final String SERVICE_NAME = "stream";

    public static final String RESOURCE_ID_PARAM_NAME = "resourceId";
    public static final String STREAM_RESOURCE_SOURCE_EXT_PARAM_NAME = "streamResourceSourceExtentionName";
    public static final String BAYEUX_STREAM_EXTENTIONS_PREFIX = "bayeux.stream.";
    public static final String DEFAULT_RESOURCE_SOURCE_NAME = "resourcesource";

    private static final Log LOG = LogFactory.getLog(StreamService.class);

    private HttpServletResponse response;

    private LinkFactory linkFactory;

    private IRequestCycle requestCycle;

    private WebRequest webRequest;

    public void setResponse(HttpServletResponse response) {
        this.response = response;
    }

    public void setLinkFactory(LinkFactory linkFactory) {
        this.linkFactory = linkFactory;
    }

    public String getName() {
        return SERVICE_NAME;
    }

    public ILink getLink(boolean post, Object parameter) {
        Defense.isAssignable(parameter, StreamServiceParameter.class, "parameter");

        StreamServiceParameter ssp = (StreamServiceParameter) parameter;

        IComponent component = ssp.getStream();

        // New since 1.0.1, we use the component to determine
        // the page, not the cycle. Through the use of tricky
        // things such as Block/InsertBlock, it is possible
        // that a component from a page different than
        // the response page will render.
        // In 1.0.6, we start to record *both* the render page
        // and the component page (if different).

        IPage activePage = requestCycle.getPage();
        IPage componentPage = component.getPage();

        Map parameters = new HashMap();

        boolean stateful = webRequest.getSession(false) != null;

        parameters.put(ServiceConstants.PAGE, activePage.getPageName());
        parameters.put(ServiceConstants.COMPONENT, component.getIdPath());
        parameters.put(ServiceConstants.CONTAINER, componentPage == activePage ? null
                : componentPage.getPageName());
        parameters.put(ServiceConstants.SESSION, stateful ? "T" : null);

        //put resourceSourceName into parameter list
        parameters.put(STREAM_RESOURCE_SOURCE_EXT_PARAM_NAME, ssp.getStreamResourceSourceExtentionName());

        // handle dynamic XHR/JSON parameters
        if (ssp.isAsync() || ssp.isJson()) {
            if (ssp.getUpdateParts() != null && ssp.getUpdateParts().length > 0)
                parameters.put(ServiceConstants.UPDATE_PARTS, ssp.getUpdateParts());

            if (ssp.isJson())
                parameters.put("json", String.valueOf(ssp.isJson()));

            // don't need to put isAsync parameter in as that is handled via http headers
        }

        parameters.put(ServiceConstants.PARAMETER, ssp.getServiceParameters());

        return linkFactory.constructLink(this, post, parameters, true);
    }

    public void service(IRequestCycle cycle) throws IOException {
        String componentId = cycle.getParameter(ServiceConstants.COMPONENT);
        String componentPageName = cycle.getParameter(ServiceConstants.CONTAINER);
        String activePageName = cycle.getParameter(ServiceConstants.PAGE);
        boolean activeSession = cycle.getParameter(ServiceConstants.SESSION) != null;

        //fetch the resource source extention name
        String streamResourceSourceExtName = cycle.getParameter(STREAM_RESOURCE_SOURCE_EXT_PARAM_NAME);

        IPage page = cycle.getPage(activePageName);

        cycle.activate(page);

        IPage componentPage = componentPageName == null ? page : cycle.getPage(componentPageName);

        IComponent component = componentPage.getNestedComponent(componentId);

        IStream stream;

        try {
            stream = (IStream) component;
        }
        catch (ClassCastException ex) {
            throw new ApplicationRuntimeException(EngineMessages.wrongComponentType(
                    component,
                    IStream.class), component, null, ex);
        }

        // Check for a StaleSession only when the session was stateful when
        // the link was created.

        if (activeSession && stream.isStateful()) {
            WebSession session = webRequest.getSession(false);

            if (session == null || session.isNew())
                throw new StaleSessionException(EngineMessages.requestStateSession(stream),
                        componentPage);
        }

        Object[] parameters = linkFactory.extractListenerParameters(cycle);

        //this will also render the response
        triggerComponent(cycle, stream, streamResourceSourceExtName, parameters);
    }

    protected void triggerComponent(IRequestCycle cycle, IStream stream, String streamResourceSourceExtName, Object[] parameters) throws IOException {
        IApplicationSpecification specification = cycle.getInfrastructure().getApplicationSpecification();

        if (streamResourceSourceExtName == null) {
            streamResourceSourceExtName = DEFAULT_RESOURCE_SOURCE_NAME;
        }
        // ask the extension for the resource
        StreamResourceSource source = (StreamResourceSource) specification.getExtension(BAYEUX_STREAM_EXTENTIONS_PREFIX + streamResourceSourceExtName, StreamResourceSource.class);

        if (LOG.isDebugEnabled()) {
            int noParameters = 0;
            if (parameters != null) {
                noParameters = parameters.length;
            }
            LOG.debug("going to process stream resource with " + noParameters + " parameter(s)");
        }

        // ask the source for the resource to be streamed
        StreamResource streamResource = source.getStreamResource(cycle, parameters);
        if (streamResource == null) {
            throw new ApplicationRuntimeException("unable to process stream, no suitable resource found.");
        }

        streamResource(streamResource);

    }

    protected void streamResource(StreamResource resource) throws IOException {
        if (LOG.isDebugEnabled()) {
            LOG.debug("streaming resource with content-type '" + resource.getContentType() + "' and content-disposition '" + resource.getContentDisposition() + "'");
        }
        response.setContentType(resource.getContentType());
        String contentDisposition = resource.getContentDisposition();
        if (contentDisposition != null) {
            response.setHeader("Content-disposition", contentDisposition);
        }
        OutputStream out = response.getOutputStream();
        resource.writeTo(out);
    }

    public void setRequest(WebRequest webRequest) {
        this.webRequest = webRequest;
    }

    public void setRequestCycle(IRequestCycle requestCycle) {
        this.requestCycle = requestCycle;
    }
}
