/*
 * Copyright 2004 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.nordija.tapestry.bayeux.stream.jfreechart;

import com.nordija.tapestry.bayeux.stream.StreamResource;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.encoders.ImageFormat;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;

/**
 * This abstract class implements the {@link #writeTo(java.io.OutputStream)} method so you don't have to concentrate
 * on how to stream the actual chart.
 * Use this if you are about to stream a JFreeChart.
 * @author Jacob von Eyben - Nordija A/S
 */
public abstract class JFreeChartResource implements StreamResource {

    private JFreeChart chart;

    public JFreeChartResource(JFreeChart chart) {
        this.chart = chart;
    }

    /**
     * <p>The image format of the chart. Can be one of the following:<br />
     * {@link ImageFormat#JPEG} or {@link ImageFormat#PNG}</p>
     *
     * @return a image format for the chart
     */
    public abstract String getImageFormat();


    /**
     * The width of the streamed chart
     *
     * @return the width
     */
    public abstract int getWidth();

    /**
     * The height of the streamed chart
     *
     * @return the height
     */
    public abstract int getHeight();

    public void writeTo(OutputStream out) throws IOException {
        ByteArrayOutputStream stream = new ByteArrayOutputStream();
        if (ImageFormat.PNG.equalsIgnoreCase(getImageFormat())) {
            ChartUtilities.writeChartAsPNG(stream, chart, getWidth(), getHeight());
        } else if (ImageFormat.JPEG.equalsIgnoreCase(getImageFormat())) {

            ChartUtilities.writeChartAsJPEG(stream, chart, getWidth(), getHeight());
        }
        out.write(stream.toByteArray());
    }

    /**
     * Returns the content type matching the defined image format returned by {@link #getImageFormat()}
     *
     * @return the content type for the jfree chart
     */
    public String getContentType() {
        return "image/" + getImageFormat();
    }
}
